﻿using System;
using System.Collections.Generic;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;

namespace CallTransfer
{
    public class BlindTransfer
    {
        private ISoftPhone softphone;
        private readonly List<string> blindTransferList;
        private readonly IPhoneCall call;
        private int currentIndex;

        public event EventHandler<VoIPEventArgs<BlindTransferState>> TransferCompleted;

        public BlindTransfer(ISoftPhone softphone, IPhoneCall call, List<string> transferList)
        {
            this.softphone = softphone;
            this.call = call;
            blindTransferList = transferList ?? new List<string>();
            currentIndex = 0;
        }

        public void Begin()
        {
            StartNextNumber();
        }

        public void End()
        {
        }

        private void BeginTransfer(string dialNumber)
        {
            IPhoneCall transferCall = softphone.CreateCallObject(call.PhoneLine, dialNumber);
            Transfer transfer = new Transfer(call, transferCall);
            transfer.TransferStateChanged += (Transfer_TransferStateChanged);
            transfer.Begin();
        }

        private void Transfer_TransferStateChanged(object sender, VoIPEventArgs<TransferState> e)
        {
            // Busy or phone error
            if (e.Item == TransferState.Error || e.Item == TransferState.Rejected)
            {
                StartNextNumber();
                return;
            }

            // call transferred successfully
            if (e.Item == TransferState.Transferred)
            {
                OnCompleted(BlindTransferState.Transferred);
                return;
            }
        }

        private void OnCompleted(BlindTransferState state)
        {
            if (TransferCompleted != null)
                TransferCompleted(this, new VoIPEventArgs<BlindTransferState>(state));
        }

        private void StartNextNumber()
        {
            // get next number
            string next = GetNextNumber();

            // no more numbers
            if (next == null)
            {
                OnCompleted(BlindTransferState.AllBusy);
                return;
            }

            // begin next transfer
            BeginTransfer(next);
        }

        private string GetNextNumber()
        {
            // end of list
            if (blindTransferList.Count <= currentIndex)
                return null;

            string nextDial = blindTransferList[currentIndex];
            currentIndex++;

            return nextDial;
        }
    }
}
